@@ -22,14 +22,15 @@
 #define THEMESDIR PKGDATADIR"/themes"
 #define THEMESFILE "/slim.theme"
 
-class Cfg {
+class Cfg
+{
 
 public:
 	Cfg();
 	~Cfg();
 
 	bool readConf(std::string configfile);
-	std::string parseOption(std::string line, std::string option);
+	bool parseOption ( std::string line );
 	const std::string& getError() const;
 	std::string& getOption(std::string option);
 	int getIntOption(std::string option);
@@ -41,14 +42,15 @@ public:
 					  char c, bool useEmpty=true);
 	static std::string Trim(const std::string &s);
 
-	std::string nextSession(std::string current);
+    std::pair<std::string,std::string> nextSession();
+	static std::string findValidRandomTheme(const std::string &set);
 
 private:
 	void fillSessionList();
 
 private:
 	std::map<std::string,std::string> options;
-	std::vector<std::string> sessions;
+    std::vector<std::pair<std::string,std::string> > sessions;
 	int currentSession;
 	std::string error;
 };
